/*
***************************************************************************

   X server for LibGGI - Basic definitions

   Copyright (C) 1998-2000 Marcus Sundberg	[marcus@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

***************************************************************************
*/

#ifndef _XGGI_STRUCT_H
#define _XGGI_STRUCT_H

#include <ggi/gg.h>
#include <ggi/ggi.h>

#include "scrnintstr.h"
#include "gcstruct.h"
#include "mipointer.h"


typedef struct _XGGIScreenBackup {

	/* Random screen procedures */

	CloseScreenProcPtr CloseScreen;
	QueryBestSizeProcPtr QueryBestSize;
	SaveScreenProcPtr SaveScreen;
	GetImageProcPtr GetImage;
	GetSpansProcPtr GetSpans;
	PointerNonInterestBoxProcPtr PointerNonInterestBox;
	SourceValidateProcPtr SourceValidate;

	/* Window Procedures */

	CreateWindowProcPtr CreateWindow;
	DestroyWindowProcPtr DestroyWindow;
	PositionWindowProcPtr PositionWindow;
	ChangeWindowAttributesProcPtr ChangeWindowAttributes;
	RealizeWindowProcPtr RealizeWindow;
	UnrealizeWindowProcPtr UnrealizeWindow;
	ValidateTreeProcPtr ValidateTree;
	PostValidateTreeProcPtr PostValidateTree;
	WindowExposuresProcPtr WindowExposures;
	PaintWindowBackgroundProcPtr PaintWindowBackground;
	PaintWindowBorderProcPtr PaintWindowBorder;
	CopyWindowProcPtr CopyWindow;
	ClearToBackgroundProcPtr ClearToBackground;
	ClipNotifyProcPtr ClipNotify;
	RestackWindowProcPtr RestackWindow;

	/* Pixmap procedures */

	CreatePixmapProcPtr CreatePixmap;
	DestroyPixmapProcPtr DestroyPixmap;

	/* Backing store procedures */

	SaveDoomedAreasProcPtr SaveDoomedAreas;
	RestoreAreasProcPtr RestoreAreas;
	ExposeCopyProcPtr ExposeCopy;
	TranslateBackingStoreProcPtr TranslateBackingStore;
	ClearBackingStoreProcPtr ClearBackingStore;
	DrawGuaranteeProcPtr DrawGuarantee;
	/*
	 * A read/write copy of the lower level backing store vector is needed now
	 * that the functions can be wrapped.
	 */
	BSFuncRec       BackingStoreFuncs;

	/* Font procedures */

	RealizeFontProcPtr RealizeFont;
	UnrealizeFontProcPtr UnrealizeFont;

	/* Cursor Procedures */

	ConstrainCursorProcPtr ConstrainCursor;
	CursorLimitsProcPtr CursorLimits;
	DisplayCursorProcPtr DisplayCursor;
	RealizeCursorProcPtr RealizeCursor;
	UnrealizeCursorProcPtr UnrealizeCursor;
	RecolorCursorProcPtr RecolorCursor;
	SetCursorPositionProcPtr SetCursorPosition;

	/* GC procedures */

	CreateGCProcPtr CreateGC;

	/* Colormap procedures */

	CreateColormapProcPtr CreateColormap;
	DestroyColormapProcPtr DestroyColormap;
	InstallColormapProcPtr InstallColormap;
	UninstallColormapProcPtr UninstallColormap;
	ListInstalledColormapsProcPtr ListInstalledColormaps;
	StoreColorsProcPtr StoreColors;
	ResolveColorProcPtr ResolveColor;
} xggiScreenBackupRec, *xggiScreenBackupPtr;


typedef struct _XGGIGCPriv {
	GCFuncs *funcs;
	GCOps *ops;
} xggiGCPrivRec, *xggiGCPrivPtr;



#define DEFAULT_PTRBUTTONS	5

struct xggiGlobalInfo {
	int             numscreens;
	int             rows, columns;	/* Screen layout */
	int             allowzap;
	int             ctrlalt_vtswitch;
	int             newvt;
	unsigned long	XGGIGeneration;
};


struct xggiVisualInfo {
	ggi_visual_t    vis;
	int             width;
	int             height;
	int             depth;
	int             stride;
	int             bitsPerPixel;

	const ggi_directbuffer *dbuf;
	char           *pfbMemory;

	ggi_mode        mode;
	VisualID        visualid;
};

struct xggiScreenInfo {
	uint32_t	ptrbuttons;
	int		rootdepth;

	ScreenPtr	screen;
	PixmapPtr	backbuffer;
	int		ismapped;

	int		numvisuals;
	VisualID	defaultVisual;
	struct xggiVisualInfo *visuals;

	int		ScreenPrivateIndex;
	int		GCPrivateIndex;
};


extern struct xggiGlobalInfo xggiInfo;
extern struct xggiScreenInfo *xggiScreens;


#define SCREEN(scr)			(xggiScreens[scr])
#define SCREEN_SCR(scr)			(SCREEN(scr).screen)
#define SCREEN_BACKBUFFER(scr)		(SCREEN(scr).backbuffer)
#define SCREEN_ISMAPPED(scr)		(SCREEN(scr).ismapped)
#define SCREEN_VIS(scr, visnr)		(SCREEN(scr).visuals[visnr])
#define SCREEN_ROOTDEPTH(scr)		(SCREEN(scr).rootdepth)
#define SCREEN_SCRIDX(scr)		(SCREEN(scr).ScreenPrivateIndex)
#define SCREEN_GCIDX(scr)		(SCREEN(scr).GCPrivateIndex)

#define SCREEN_SCRPRIV(scr)		SCREEN_SCR(scr)->devPrivates[SCREEN_SCRIDX(scr)]
#define SCREEN_SCRBACKUP(scr)		((xggiScreenBackupPtr)SCREEN_SCRPRIV(scr).ptr)


#define GC_SCR(gc)			(gc->pScreen)
#define GCPRIV(scr, gc)			(gc)->devPrivates[SCREEN_GCIDX(scr)]
#define GCBACKUP(scr, gc)		((xggiGCPrivPtr)GCPRIV(scr, gc).ptr)
#define GCBACKUP_OPS(scr, gc)		GCBACKUP(scr, gc)->ops
#define GCBACKUP_FUNCS(scr, gc)		GCBACKUP(scr, gc)->funcs



#define VISUAL(scr, visnr)		(SCREEN_VIS(scr, visnr))
#define VISUAL_ID(scr, visnr)		(SCREEN_VIS(scr, visnr).visualid)
#define VISUAL_WIDTH(scr, visnr)	(SCREEN_VIS(scr, visnr).width)
#define VISUAL_HEIGHT(scr, visnr)	(SCREEN_VIS(scr, visnr).height)
#define VISUAL_DEPTH(scr, visnr)	(SCREEN_VIS(scr, visnr).depth)
#define VISUAL_STRIDE(scr, visnr)	(SCREEN_VIS(scr, visnr).stride)
#define VISUAL_VIS(scr, visnr)		(SCREEN_VIS(scr, visnr).vis)
#define VISUAL_BPP(scr, visnr)		(SCREEN_VIS(scr, visnr).bitsPerPixel)
#define VISUAL_MODE(scr, visnr)		(SCREEN_VIS(scr, visnr).mode)
#define VISUAL_SCREEN(scr, visnr)	(SCREEN_SCR(scr))
#define VISUAL_BACKBUFFER(scr, visnr)	(SCREEN_BACKBUFFER(scr))
#define VISUAL_ISMAPPED(scr, visnr)	(SCREEN_ISMAPPED(scr))


#define MASTER_SCREEN_NR		(0)
#define MASTER_VISUAL_NR		(0)
#define MASTER_SCREEN			SCREEN(MASTER_SCREEN_NR)
#define MASTER_VISUAL(scr)		VISUAL(scr, MASTER_VISUAL_NR)
#define MASTER_VISUAL_SCREEN(scr)	VISUAL_SCREEN(scr, MASTER_VISUAL_NR)
#define MASTER_VISUAL_ID(src)		VISUAL_ID(scr, MASTER_VISUAL_NR)
#define MASTER_VISUAL_WIDTH(scr)	VISUAL_WIDTH(scr, MASTER_VISUAL_NR)
#define MASTER_VISUAL_HEIGHT(scr)	VISUAL_HEIGHT(scr, MASTER_VISUAL_NR)
#define MASTER_VISUAL_DEPTH(scr)	VISUAL_DEPTH(scr, MASTER_VISUAL_NR)
#define MASTER_VISUAL_STRIDE(scr)	VISUAL_STRIDE(scr, MASTER_VISUAL_NR)
#define MASTER_VISUAL_VIS(scr)		VISUAL_VIS(scr, MASTER_VISUAL_NR)
#define MASTER_VISUAL_BPP(scr)		VISUAL_BPP(scr, MASTER_VISUAL_NR)
#define MASTER_VISUAL_MODE(scr)		VISUAL_MODE(scr, MASTER_VISUAL_NR)
#define MASTER_VISUAL_BACKBUFFER(scr)	VISUAL_BACKBUFFER(scr, MASTER_VISUAL_NR)
#define MASTER_VISUAL_ISMAPPED(scr)	VISUAL_ISMAPPED(scr, MASTER_VISUAL_NR)

#define PIXMAP_HAS_MASTER_VISUAL_BPP(pixmap, scr)	(pixmap)->drawable.bitsPerPixel == MASTER_VISUAL_BPP(scr)
#define WINDOW_HAS_MASTER_VISUAL_BPP(window, scr)	(window)->drawable.bitsPerPixel == MASTER_VISUAL_BPP(scr)
#define DRAWABLE_HAS_MASTER_VISUAL_BPP(drawable, scr)	(drawable)->bitsPerPixel == MASTER_VISUAL_BPP(scr)

/* convinient Directbuffer macros */
#define DBUF_PROLOGUE(scr)						\
	ggiResourceRelease(MASTER_VISUAL(scr).dbuf->resource);

#define DBUF_EPILOGUE(scr)						\
	ggiFlush(MASTER_VISUAL_VIS(scr));				\
	if (ggiResourceAcquire(MASTER_VISUAL(scr).dbuf->resource,	\
			GGI_ACTYPE_WRITE | GGI_ACTYPE_READ) != 0)	\
	{								\
		FatalError("Unable to re-acquire DirectBuffer\n");	\
	}


#define TVAL2TIME(tv)	((tv).tv_sec*1000 + (tv).tv_usec/1000)

#ifndef MIN
#define MIN(a,b)	((a) < (b) ? (a) : (b))
#endif
#ifndef MAX
#define MAX(a,b)	((a) > (b) ? (a) : (b))
#endif


#endif				/* _XGGI_STRUCT_H */
